'use strict';

angular.module('sw', ['ngSanitize']).directive('swGrid', ['$compile', '$filter', '$sanitize', function($compile, $filter, $sanitize) {
    var sortIcon = '<i class="icon sprite sort"></i>';

    return {
        restrict: 'E',
        replace: true,
        scope: {
            'options': '='
        },
        template: [
            '<table ng-class="options.tableCls">',
                '<thead>',
                    '<tr>',
                        '<th class="index wide-fix">{{options.indexTitle}}</th>',
                        '<th ng-repeat="column in options.columns" ng-click="updateSorted($index)" ng-class="{selected: !!selectionState($index)}" class="{{column.cls}}" sortable="{{column.name}}" sorted="{{selectionState($index)}}">',
                            '<div>',
                                '<span>{{column.header}}</span>',
                                '<span iconHolder ng-bind-html-unsafe="getSortIcon(column)"></span>',
                            '</div>',
                        '</th>',
                    '</tr>',
                '</thead>',
                '<tbody>',
                    '<tr ng-repeat="row in options.data">',
                        '<td>{{$index + 1}}.</td>',
                        '<td ng-class="{selected: isSelected($index)}" ng-repeat="column in options.columns" ng-bind-html-unsafe="insertTd($index, row)"></td>',
                    '</tr>',
                '</tbody>',
            '</table>'
        ].join(""),
        compile: function(tElement, tAttr){
            var sortable = true;

            if(eval(tAttr.sortable) === false){
                sortable = false;
                tElement.find('th').attr('sortable', null);
                tElement.find('th').attr('sorted', null);
                tElement.find('th div span[iconHolder]').remove();
            }

            return {
                pre: function (scope, iElement) {

                    if (sortable){
                        scope.sortedDirection = "desc";
                        scope.sortedColIndex = _.indexOf(scope.options.columns, _.find(scope.options.columns, function(column){
                            return column.name === scope.options.defaultSorted;
                        }));

                        scope.getSortIcon = function(column){
                            if (column.sortable !== false){
                                return sortIcon;
                            }
                            return '';
                        }

                        scope.selectionState = function(i){
                            if (scope.sortedColIndex !== i) {
                                return '';
                            };

                            return scope.sortedDirection;
                        }

                        scope.updateSorted = function(i){
                            if (scope.sortedColIndex === i){
                                scope.sortedDirection = scope.sortedDirection === 'desc' ? 'asc' : 'desc'
                            }
                            else {
                                scope.sortedDirection = "desc";
                            }
                            scope.sortedColIndex = i;
                        }
                    }
                },
                post: function(scope){
                    scope.insertTd = function(index, row){
                        var column =  scope.options.columns[index],
                            name = column.name,
                            tpl = column.tpl || '{{value}}';

                        tpl = angular.isFunction(tpl) ? tpl : (function(tpl){
                            return function(filter, value, row) {
                                return tpl.replace('{{value}}', value);
                            }
                        })(tpl);

                        return tpl($filter, row[name], row);
                    };
                }
            }
        }
    }
}]);
